/**
 * A parser for .torrent files encoded with Bencode <https://en.wikipedia.org/wiki/Bencode>.
 */

import { TextDecoder } from 'util';

const Decoder = new TextDecoder('utf-8');

/** Takes a Bencoded-encoded file, parses it at the given starting position and returns a JSON object, or rejects on error. */
function bpParse(dv: DataView, posIn: number = 0): { obj: any, pos: number } {
  let pos = posIn;
  let obj: any;
  const header = dv.getUint8(pos); pos += 1;
  switch (header) {
    case 0x64: { //'d' - dictionary (key-value object)
      obj = {};
      do {
        //read key
        const out1 = bpParse(dv, pos);
        pos = out1.pos;
        if (typeof out1.obj !== 'string') {
          throw new Error(`Expected dictionary key to be string but it is "${typeof out1.obj}".`);
        }
        //read value
        const out2 = bpParse(dv, pos);
        pos = out2.pos;
        obj[out1.obj] = out2.obj;
      } while (dv.getUint8(pos) !== 0x65); //'e' - end
      pos += 1;
      break;
    }
    case 0x6C: { //'l' - list (array)
      obj = [];
      do {
        //read entry
        const out = bpParse(dv, pos);
        pos = out.pos;
        obj.push(out.obj);
      } while (dv.getUint8(pos) !== 0x65); //'e' - end
      pos += 1;
      break;
    }
    case 0x69: { //'i' - integer
      let curChar = dv.getUint8(pos); pos += 1;
      let curNumber = 0;
      while (curChar !== 0x65) { //'e' - end
        if (curChar < 0x30 || curChar > 0x39) {
          throw new Error('Unexpected int char during bencode parsing.');
        }
        curNumber *= 10;
        curNumber += curChar - 0x30;
        curChar = dv.getUint8(pos); pos += 1;
      }
      obj = curNumber;
      break;
    }
    case 0x30: case 0x31: case 0x32: case 0x33: case 0x34:
    case 0x35: case 0x36: case 0x37: case 0x38: case 0x39: { //string prefixed by length
      //read string length
      let curChar: number = header;
      let strLen = 0;
      while (curChar !== 0x3A) { //':' - integer delimiter, beginning of string
        if (curChar < 0x30 || curChar > 0x39) {
          throw new Error('Unexpected string length during bencode parsing.');
        }
        strLen *= 10;
        strLen += curChar - 0x30;
        curChar = dv.getUint8(pos); pos += 1;
      }
      //read string
      obj = Decoder.decode(new DataView(dv.buffer, pos, strLen));
      pos += strLen;
      break;
    }
    default:
     throw new Error('Unexpected leading char during bencode parsing.');
  }
  return { obj, pos };
}

export default function parseBencode(dv: DataView): any {
  return bpParse(dv).obj;
}
